<h2>Scope</h2>
<table class="table table-hover">
  <tbody>
    <tr>
      <td>Goals</td>
      <td>{{ goals_form.id|length }}</td>
    </tr>
    <tr>
      <td>Attributes</td>
      <td>{{ attributes_form.id|length }}</td>
    </tr>
    <tr>
      <td>Metrics</td>
      <td>{{ metrics_form.id|length }}</td>
    </tr>
  </tbody>
</table>
