{# Modal dialog for importing the definition of a model in a supported JSON format #}
<!-- Modal: Import -->
<div class="modal fade" id="importModal" role="dialog">
    <div class="modal-dialog">
    <!-- Modal content-->
        <div class="modal-content">
          <form id="qmodel_import" method="POST" action="import" enctype="multipart/form-data">
          {% csrf_token %}
          <div class="modal-header">
              <div class="modal-title">
                  Import quality model file
              </div>
            <button type="button" class="close" data-dismiss="modal">&times;</button>
          </div>
          <div class="modal-body">
             <div class="input-group">
               <span class="input-group-addon"><i class="fa fa-arrow-circle-o-up"></i></span>
               <input type="file" name="imported_file" placeholder="Select JSON file" accept="application/json" required=True class="form-control">
             </div>
          </div>
          <div class="modal-footer">
            <button type="submit" class="btn btn-primary btn-sm">Import</button>
          </div>
        </form>
        </div>
    </div>
</div>
