{% extends 'base.html' %}

{% block title %}Visualize Quality Models{% endblock %}

{% block head %}
    {{ block.super }}
{% endblock %}

{% block body %}

{% if errors %}
<div class="container-fluid">
    <div class="alert alert-danger">
        <a href="#" class="close" data-dismiss="alert">&times;</a>
        <strong>Errors:</strong> {{ errors }}
    </div>
</div>
{% endif %}

<!-- Start: Visualization Config Editor -->
<div class="container-fluid">
  {% include "prosoul/visualization_config.html" %}
</div>
<!-- End: Visualization Config Editor -->


{% endblock %}
