<div class="row">
  <div class="col-sm-6">
    <h2>Attributes</h2>
  </div>
  <div class="col-sm-6">
    <div class="button-group">
      <button type="submit" class="btn btn-primary btn-sm" id="add-attribute-btn" data-toggle="modal" data-target="#addAttributeModal">
        <i class="fa fa-plus"></i> Add<span class="sr-only"> goal</span>
      </button>
      <button type="submit" class="btn btn-primary btn-sm" id="edit-attribute-btn" data-toggle="modal" data-target="#addAttributeModal">
        <i class="fa fa-pencil"></i> Edit<span class="sr-only"> goal</span>
      </button>
    </div>
  </div>
</div>

<div class="row">
  <div class="col-sm-12">
    <form action="./select_attribute" method="post">
      <fieldset id="select-attribute">
        {% csrf_token %}
        {% for field in attributes_form.state_fields %}
        {{ field }}
        {% endfor %}
        {{ attributes_form.name.errors }}
        <div class="form-group">
          {{ attributes_form.id }}
        </div>
      </fieldset>
    </form>
  </div>
</div>
