# Generated by Django 2.0.4 on 2018-04-25 04:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('prosoul', '0002_auto_20180307_2051'),
    ]

    operations = [
        migrations.AddField(
            model_name='metricdata',
            name='params',
            field=models.CharField(blank=True, max_length=1024, null=True),
        ),
    ]
