<!DOCTYPE html>
<html lang="en">
    <head>
        {% block head %}
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
        <title>Prosoul {% block title %}{% endblock %}</title>
        <link rel="stylesheet" type="text/css" href="//cdn.datatables.net/1.10.16/css/jquery.dataTables.css">
        <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0-beta.2/css/bootstrap.min.css" integrity="sha384-PsH8R72JQ3SOdhVi3uxftmaW6Vc51MKb0q5P2rRUpPvrszuE4W1povHYgTpBfshb" crossorigin="anonymous">
        <script
            src="https://code.jquery.com/jquery-3.2.1.slim.min.js"
            integrity="sha256-k2WSCIexGzOj3Euiig+TlR8gA0EmPjuc79OEeY5L45g="
            crossorigin="anonymous"></script>
        <script type="text/javascript" charset="utf8" src="//cdn.datatables.net/1.10.16/js/jquery.dataTables.js"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.12.3/umd/popper.min.js" integrity="sha384-vFJXuSJphROIrBnz7yo7oB41mKfc8JzQZiCq4NCceLEaO4IHwicKwpJf9c9IpFgh" crossorigin="anonymous"></script>
        <script src="https://maxcdn.bootstrapcdn.com/bootstrap/4.0.0-beta.2/js/bootstrap.min.js" integrity="sha384-alpBpkh1PFOepccYVYDB4do5UnbKysX5WZXm3XxPqe5iKTfUKjNkCk9SaVuEZflJ" crossorigin="anonymous"></script>
        <script src="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datepicker/1.7.1/js/bootstrap-datepicker.min.js" integrity="sha256-TueWqYu0G+lYIimeIcMI8x1m14QH/DQVt4s9m/uuhPw=" crossorigin="anonymous"></script>
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/bootstrap-datepicker/1.7.1/css/bootstrap-datepicker.min.css" integrity="sha256-I4gvabvvRivuPAYFqevVhZl88+vNf2NksupoBxMQi04=" crossorigin="anonymous"/>
        <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css">
        {% endblock %}
    </head>
    <body>
        <nav class="navbar navbar-expand-lg navbar-light bg-light">
            <a href="/" class="navbar-brand">
                <img src="/static/img/prosoul.svg" width="30" height="30" class="d-inline-block align-top" alt="">Prosoul
            </a>

            <ul class="navbar-nav">
              <li class="nav-item">
                  <a class="nav-link {% if active_page == 'viewer' %} active {% endif %}" href="viewer">View</a>
              </li>
              <li class="nav-item">
                  <a class="nav-link {% if active_page == 'editor' %} active {% endif %}" href="editor">Edit</a>
              </li>
              <li class="nav-item">
                  <a class="nav-link {% if active_page == 'visualize' %} active {% endif %}" href="visualize">Visualize</a>
              </li>
              <li class="nav-item">
                  <a class="nav-link {% if active_page == 'assess' %} active {% endif %}" href="assess">Assess</a>
              </li>
              <li class="nav-item">
                  <a class="nav-link {% if active_page == 'admin' %} active {% endif %}" href="/admin">Admin (admin/admin for login)</a>
              </li>
            </ul>
        </nav>
        <div class="container-fluid">
            {% if err != None %}
            <div class="alert alert-warning alert-dismissible fade show" role="alert">
                <strong>Ooos!</strong> {{err}}
                <button type="button" class="close" data-dismiss="alert" aria-label="Close">
                    <span aria-hidden="true">&times;</span>
                </button>
            </div>
            {% endif %}
        {% block body %}{% endblock %}
        </div>
        <footer class="footer fixed-bottom container-fluid bg-4 text-center" style="margin-bottom: 20px;">
          Powered by <a href="https://grimoirelab.github.io">GrimoireLab</a>
        </footer>
    </body>
</html>
