# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-03-07 20:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('prosoul', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attribute',
            name='name',
            field=models.CharField(max_length=200),
        ),
        migrations.AlterField(
            model_name='goal',
            name='name',
            field=models.CharField(max_length=200),
        ),
    ]
