
{% if assessment %}
<div class="row">
  <div class="col-sm-12">
    <h1>Projects Assessment</h1>
    {{ assessment | safe}}
  </div>
</div>
<div class="row">
    <div class="col-sm-12">
          Each metric assess has 5 levels according to thresholds defined
          in the quality model.
    </div>
</div>
{% endif %}


<div class="row">
  <div class="col-sm-12">
      <form action="./create_assessment" method="post">
          <fieldset id="config-vis" {% if assessment %} hidden {% endif %}>
            {% csrf_token %}
            <div class="modal-body">
              <div class="input-group"><span class="input-group-addon">
                {{ assess_config_form.quality_model.label }}</span>{{ assess_config_form.quality_model }}</div>
              <div class="input-group"><span class="input-group-addon">
                {{ assess_config_form.es_url.label }}</span>{{ assess_config_form.es_url }}</div>
              <div class="input-group"><span class="input-group-addon">
                {{ assess_config_form.es_index.label }}</span>{{ assess_config_form.es_index }}</div>
              <div class="input-group"><span class="input-group-addon">
                {{ assess_config_form.backend_metrics_data.label }}</span>{{ assess_config_form.backend_metrics_data }}</div>
            </div>
            <div class="form-group">
              <button type="submit" class="btn btn-primary btn-sm" id="create-assess-btn"><span>
                <i class="fa fa-check"></i></span> Create<span class="sr-only"> Assessment<span></button>
            </div>
          </fieldset>
      </form>
  </div>
</div>
