<div class="row">

  <div class="col-md-12">
    <div class="row">
      <form class="form-inline" action="./select_qmodel" method="post">
        {% csrf_token %}
        {% for field in qmodels_form.state_fields %}
          {{ field }}
        {% endfor %}
        {{ qmodels_form.name.errors }}
        <label for="id_id"><h1>QualityModel:</h1></label>
        {{ qmodels_form.id }}
      </form>
      <div class="button-group" style="margin:20px">
        <button type="submit" class="btn btn-primary btn-sm" id="add-qmodel-btn" data-toggle="modal" data-target="#addQualityModelModal">
          <i class="fa fa-plus"></i> Add <span class="sr-only">qmodel</span>
        </button>
        <button type="submit" class="btn btn-primary btn-sm" id="edit-qmodel-btn" data-toggle="modal" data-target="#addQualityModelModal">
          <i class="fa fa-edit"></i> Edit <span class="sr-only">qmodel</span>
        </button>
      </div>

      <div class="button-group" style="margin:20px">
        <button type="submit" class="btn btn-default btn-sm" data-toggle="modal" data-target="#importModal">
          <i class="fa fa-upload"></i> Import <span class="sr-only">qmodel</span>
        </button>
        <button type="submit" class="btn btn-default btn-sm" data-toggle="modal" data-target="#exportModal">
          <i class="fa fa-download"></i> Export <span class="sr-only">qmodel</span>
        </button>
      </div>
  </div>

  </div>

</div>
