{# Modal dialog for downloading the definition of a model in GrimoireLab JSON format #}
<!-- Modal: Download -->
<div class="modal fade" id="exportModal" role="dialog">
    <div class="modal-dialog">
    <!-- Modal content-->
        <div class="modal-content">
          <form id="qmodel_download" method="POST" action="export" enctype="multipart/form-data">{% csrf_token %}
          <div class="modal-header">
            <div class="modal-title">Download quality model file</div>
            <button type="button" class="close" data-dismiss="modal">&times;</button>
          </div>
          <div class="modal-body">
                  <div class="input-group">
                      <span class="input-group-addon"><i class="fa fa-globe"></i></span>
                      {{ qmodels_form.id }}
                  </div>
          </div>
          <div class="modal-footer">
            <button type="submit" class="btn btn-primary btn-sm">Download</button>
          </div>
        </form>
        </div>
    </div>
</div>
